﻿using System.Security.Cryptography;
using System.Text;

namespace anbs_cp.Classes.Encrypt;

/// <summary>
/// Класс для шифровки строк
/// </summary>
public static class StringEncrypt
{
    /// <summary>
    /// Получение ключа из строки
    /// </summary>
    /// <param name="s">Ключ-строка</param>
    /// <param name="salt">Хэш-ключ</param>
    /// <returns>Ключ</returns>
    private static byte[] KeyFromString (string s, byte[] salt)
    {
        // Создаю хэшер
        using Rfc2898DeriveBytes hasher = new(s, salt, 1000, HashAlgorithmName.SHA256);

        // Получаю ключ
        return hasher.GetBytes(32);
    }

    /// <summary>
    /// Метод для шифрования строки <paramref name="text"/>
    /// </summary>
    /// <param name="text">Строка, которая должна быть зашифрована</param>
    /// <param name="key">Ключ</param>
    /// <returns>Этот статический метод возвращает зашифрованную строку <paramref name="text"/></returns>
    public static string Encrypt (string text, string key)
    {
        // Создаю криптограф
        using Aes aes = Aes.Create();

        // Получаю ключ
        aes.Key = KeyFromString(key, aes.IV);

        // Открываю поток
        using MemoryStream ms = new();

        // Пишу данные в поток
        ms.Write(aes.IV);

        // Создаю шифрованный поток
        using (CryptoStream cs = new(ms, aes.CreateEncryptor(), CryptoStreamMode.Write, true))
            // Пишу данные в него
            cs.Write(Encoding.UTF8.GetBytes(text));

        // Возвращаю зашифрованный текст
        return Convert.ToBase64String(ms.ToArray());
    }

    /// <summary>
    /// Метод для дешифрования строки <paramref name="text"/>
    /// </summary>
    /// <param name="text">Строка, которая должна быть дешифрована</param>
    /// <param name="key">Ключ</param>
    /// <returns>Этот статический метод возвращает дешифрованную строку <paramref name="text"/></returns>
    public static string Decrypt (string text, string key)
    {
        // Открываю поток в памяти
        using MemoryStream ms = new(Convert.FromBase64String(text));

        // Задаю ключ
        byte[] iv = new byte[16];

        // Читаю его
        _ = ms.Read(iv);

        // Создаю криптограф
        using Aes aes = Aes.Create();

        // Получаю ключ
        aes.Key = KeyFromString(key, iv);

        // присваиваю ключ
        aes.IV = iv;

        // Создаю поток дешифратора
        using CryptoStream cs = new(ms, aes.CreateDecryptor(), CryptoStreamMode.Read, true);

        // Задаю поток итогового текста
        using MemoryStream output = new();

        // Копирую данные в выходной поток
        cs.CopyTo(output);

        // Вывожу расшифрованный текст
        return Encoding.UTF8.GetString(output.ToArray());
    }

    /// <summary>
    /// Декодирует зашифрованную строку в HTML-пригодный формат
    /// </summary>
    /// <param name="text">Зашифрованная строка</param>
    /// <returns>Этот статический метод возвращает дешифрованную строку <paramref name="text"/></returns>
    public static string Base64UrlEncode(string text) => text.TrimEnd('=').Replace('+', '-').Replace('/', '_');

    /// <summary>
    /// Раскодирует из декодированной строки в HTML-пригодный формат
    /// </summary>
    /// <param name="text">Декодированная строка</param>
    /// <returns>Этот статический метод возвращает шифрованную строку <paramref name="text"/></returns>
    public static string Base64UrlDecode(string text)
    {
        string result = text.Replace('_', '/').Replace('-', '+');
        switch (result.Length % 4)
        {
            case 2:
                result += "==";
                break;
            case 3:
                result += "=";
                break;
        }
        return result;
    }
}